/* 
**  locking.c -- locking functions for mod clamav
**
**  (c) 2003-2009 Prof. Dr. Andreas Mueller, Beratung und Entwicklung
**
**  This module is distributed under the GNU General Public License (GPL).
**  See the file COPYING for details.
**
**  $Id: locking.c,v 1.1 2009/04/10 23:32:37 afm Exp $
*/ 
#include <mod_clamav.h>

/* set up the mutex lock, delayed until the first request happens */
static void	mod_clamav_mutex(mod_clamav_config_rec *rec, request_rec *r) {
    mode_t		mask;

    /* skip creating the lock if it already exists */
    if (NULL != rec->mutex)
	return;

    /* create the mutex lock */
    mask = umask(077);
    if (APR_SUCCESS != apr_proc_mutex_create(&rec->mutex,
	(rec->mutexname) ? rec->mutexname : "/var/tmp/clamav.lock",
	APR_LOCK_DEFAULT, rec->pool)) {
	ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
	    "[%d] cannot create mutex %s", (int)getpid(),
	    (rec->mutexname) ? rec->mutexname : "/var/tmp/clamav.lock");
	rec->mutex = NULL;
    }
    umask(mask);
}

/* functions to lock and unlock */
void	mod_clamav_lock(mod_clamav_config_rec *rec, request_rec *r) {
    mod_clamav_mutex(rec, r);
    if (NULL != rec->mutex)
	apr_proc_mutex_lock(rec->mutex);
} /* clamav_lock */

void	mod_clamav_unlock(mod_clamav_config_rec *rec, request_rec *r) {
    mod_clamav_mutex(rec, r);
    if (NULL != rec->mutex)
	apr_proc_mutex_unlock(rec->mutex);
} /* clamav_unlock */

