/* 
**  mod_clamav.h -- common header for the mod_clamav virus scanning module
**
**  (c) 2003-2009 Dr. Andreas Mueller, Beratung und Entwicklung
**
**  This module is distributed under the GNU General Public License (GPL).
**  See the file COPYING for details.
**
**  $Id: mod_clamav.h,v 1.2 2009/04/10 23:37:29 afm Exp $
*/ 
#ifndef MOD_CLAMAV_H
#define MOD_CLAMAV_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "http_log.h"
#include "http_core.h"
#include "ap_config.h"
#include "ap_compat.h"
#include <apr.h>
#include <apr_strings.h>
#include <apr_time.h>
#ifdef HAVE_CLAMAV_H
#include <clamav.h>
#endif /* HAVE_CLAMAV_H */
#ifdef HAVE_FCNTL_H 
#include <fcntl.h>
#endif /* HAVE_FCNTL_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif /* HAVE_SYS_TIME_H */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif /* HAVE_SYS_RESOURCE_H */
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif /* HAVE_SYS_SOCKET */
#ifdef HAVE_SYS_UN_H
#include <sys/un.h>
#endif /* HAVE_SYS_UN_H */
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif /* HAVE_NETINET_IN_H */
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif /* HAVE_ARPA_INET_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif /* HAVE_CTYPE_H */
#ifdef HAVE_ALLOCA_H
#include <alloca.h>
#endif /* HAVE_ALLOCA_H */
#ifdef HAVE_REGEX_H
#include <regex.h>
#endif /* HAVE_REGEX_H */

#ifndef CLAMAV_DEBUG
#define	CLAMAV_DEBUG	0
#endif /* CLAMAV_DEBUG */
#if CLAMAV_DEBUG
#define	mod_clamav_ap_log_rerror(x, args...) ap_log_rerror(x, ##args)
#else
#define mod_clamav_ap_log_rerror(args...)
#endif

#define	MOD_CLAMAV_LOCAL	0
#define	MOD_CLAMAV_DAEMON	1

#define MOD_CLAMAV_NOTE_PREFIX	"clamav:"
#define MOD_CLAMAV_STATUS_NOTE	MOD_CLAMAV_NOTE_PREFIX	"status"
#define MOD_CLAMAV_DETAILS_NOTE	MOD_CLAMAV_NOTE_PREFIX	"details"
#define MOD_CLAMAV_VIRUSNAME_NOTE	MOD_CLAMAV_NOTE_PREFIX	"virusname"
#define MOD_CLAMAV_LONGSTATUS_NOTE	MOD_CLAMAV_NOTE_PREFIX	"longstatus"

#define MOD_CLAMAV_PASSED_STATUS	"passed"
#define MOD_CLAMAV_BYPASSED_STATUS	"bypassed"
#define MOD_CLAMAV_ABORTED_STATUS	"aborted"
#define MOD_CLAMAV_INFECTED_STATUS	"INFECTED"
#define MOD_CLAMAV_FAILED_STATUS	"failed"

module AP_MODULE_DECLARE_DATA clamav_module;

/* the clamav statistics context */
#define	VIRUS_LIST_LENGTH	10
#define VIRUS_NAME_LENGTH	64
#define	VIRUS_URI_LENGTH	256
#define	VIRUS_REQ_LENGTH	64

#define MATCH_SAFE_URI	0
#define MATCH_SAFE_HOST	1

typedef struct mod_clamav_virusinfo {
    pid_t	pid;	/* the process id of the apache process that detected */
			/* the virus */
    char	uri[VIRUS_URI_LENGTH];	/* the uri that contained the virus */
    char	req[VIRUS_REQ_LENGTH];	/* request that contained the virus */
    char	virus[VIRUS_NAME_LENGTH];	/* the virus name */
    off_t	size;	/* the size of the data checked */
    apr_time_t	when;	/* the time of the virus event */
} mod_clamav_virusinfo;

typedef struct mod_clamav_stats {
    unsigned long	requests;	/* the number of requests */
    unsigned long	checked;	/* requests actally checked */
    unsigned long	aborted;	/* requests aborted prematurely */
    unsigned long long	totalsize;	/* total size of data transfered */
    off_t		maxsize;	/* maximum request checked */
    off_t		maxvirussize;	/* maximum virus file size */
    double		cpu;		/* cpu time used for virus checks */
    unsigned long	viruses;	/* number of viruses find */
    mod_clamav_virusinfo	lastviruses[VIRUS_LIST_LENGTH];	/* list of viruses */
    int			last;		/* last entry index in lastviruses */
    unsigned long	reloads;	/* number of database reloads (local) */
} mod_clamav_stats;

/* we don't want to duplicate the clamav trie, so we keep it in a separate */
/* structure								   */
typedef struct cl_daemon_t {
    struct sockaddr	*s;
    int			len;
    int			domain;
} cl_daemon_t;

typedef struct cl_local_t {
    struct cl_engine	*engine;
    time_t		lastreload;
} cl_local_t;

typedef union mod_clamav_common {
    cl_local_t	*local;
    cl_daemon_t	*daemon;
} mod_clamav_common;

typedef struct mod_clamav_safepattern {
	char	mode;		/* type of pattern			*/
	char	tag[31];	/* short name of pattern		*/
	char	pattern[16];	/* pattern				*/
	char	mask[16];	/* bit mask				*/
} mod_clamav_safepattern;

typedef struct mod_clamav_safeuri {
	char	pattern[VIRUS_URI_LENGTH];
	ap_regex_t	*regex;
	int	matchtype;
} mod_clamav_safeuri;

/* the clamav module configuration					*/
typedef struct mod_clamav_config_rec {
    char		*tmpdir;
    int			mode;		/* mod_clamav mode: local or daemon */
    const char		*dbdir;		/* local: where are virus patterns */
    int			port;		/* daemon: clamd port number */
    char		*socket;	/* daemon: unix domain socket path */
    int			maxfiles;
    long long		maxfilesize;
    int			maxreclevel;
    int			trickle_interval;
    int			trickle_size;
    int			sizelimit;
    int			reload_interval;
    int			acceptdaemonproblem;
    int			extendedlogging;
    mode_t		perms;
    apr_table_t		*safetypes;	/* types safe to bypass av checking */
    apr_array_header_t	*safepatterns;
    apr_array_header_t	*safeuris;
    char		*message;
    /* virus scanning engine */
    mod_clamav_common	*common;
    /* shared data for statistics */
    apr_shm_t		*shm;
    char		*shmname;
    mod_clamav_stats	*stats;
    /* mutex to mediate access to shared data */
    apr_proc_mutex_t	*mutex;
    char		*mutexname;
    /* the pool to use for long term allocations */
    apr_pool_t		*pool;
} mod_clamav_config_rec;

/* the clamav filter context						*/
typedef struct mod_clamav_ctx {
    apr_bucket_brigade	*bb;
    char		*filename;
    int			file;
    time_t		last_trickle;
    long long		trickle_offset;
    mod_clamav_config_rec	*module;
    int			bypass;
    long long		bytes;
} mod_clamav_ctx;

/* get the module configuration record */
extern mod_clamav_config_rec	*mod_clamav_get_module_config(request_rec *r);

/* functions to lock and unlock */
extern void	mod_clamav_lock(mod_clamav_config_rec *rec, request_rec *r);
extern void	mod_clamav_unlock(mod_clamav_config_rec *rec, request_rec *r);

/* set request notes */
#define mod_clamav_set_bypassed_note(rec, f, reason)			\
	mod_clamav_set_status_note(rec, f, MOD_CLAMAV_BYPASSED_STATUS, 	\
		reason, NULL)

extern void	mod_clamav_virus_record(mod_clamav_stats *stats,
	const char *uri, const char *req, const char *virus, off_t size);
extern void	mod_clamav_set_status_note(mod_clamav_config_rec *rec,
	ap_filter_t *f, const char *status, const char *details,
	const char *virusname);
extern int	mod_clamav_safe_to_bypass(mod_clamav_config_rec *rec,
	ap_filter_t *f);
extern void	mod_clamav_shm(mod_clamav_config_rec *rec, request_rec *r);
extern int	mod_clamav_engine_init(mod_clamav_config_rec *rec,
	ap_filter_t *f);
extern int	mod_clamav_engine_ready(mod_clamav_config_rec *rec,
	ap_filter_t *f);
extern int	mod_clamav_virus_info(mod_clamav_config_rec *rec,
	ap_filter_t *f, const char *virname);
extern void	mod_clamav_display_virus(mod_clamav_config_rec *rec,
	request_rec *r, mod_clamav_virusinfo *v);
extern int	mod_clamav_handler(request_rec *r);
extern apr_status_t	mod_clamav_filter(ap_filter_t *f,
	apr_bucket_brigade *bb);

extern double	mod_clamav_cpu(mod_clamav_config_rec *rec,
	struct rusage *begin, struct rusage *end);
extern apr_status_t	clamav_filter(ap_filter_t *f, apr_bucket_brigade *bb);
extern void	*mod_clamav_create_dir_config(apr_pool_t *p, char *d);
extern void	*mod_clamav_merge_dir_config(apr_pool_t *p, void *pp, void *cp);

extern const command_rec	mod_clamav_cmds[];

#endif /* HAVE_MOD_CLAMAV_H */
