/* 
**  shmem.c -- shared memory for mod_clamav module
**
**  (c) 2003-2009 Dr. Andreas Mueller, Beratung und Entwicklung
**
**  This module is distributed under the GNU General Public License (GPL).
**  See the file COPYING for details.
**
**  $Id: shmem.c,v 1.2 2009/04/10 23:37:29 afm Exp $
*/ 
#include <mod_clamav.h>

/* set up shared memory, delayed until the first request happens */
void	mod_clamav_shm(mod_clamav_config_rec *rec, request_rec *r) {
    apr_status_t	rc;
    mode_t		mask;

    /* skip creating of shared memory */
    if (NULL != rec->shm)
	return;

    /* create share memory */
    mod_clamav_lock(rec, r);
    rc = apr_shm_attach(&rec->shm,
	(rec->shmname) ? rec->shmname : "/var/tmp/clamav.shm", rec->pool);
    if (APR_SUCCESS != rc) {
	/* simply attaching to the shared memory didn't work so we must	*/
	/* create it first						*/
	mask = umask(077);
	rc = apr_shm_create(&rec->shm, sizeof(mod_clamav_stats),
	    (rec->shmname) ? rec->shmname : "/var/tmp/clamav.shm", rec->pool);
	umask(mask);
	if (APR_SUCCESS != rc) {
	    fprintf(stderr, "%s:%d: cannot create shared memory %s: "
		"statistics will not be available\n", __FILE__, __LINE__,
		(rec->shmname) ? rec->shmname : "/var/tmp/clamav.shm");
	    rec->shm = NULL;
	} else {
	    /* since we are the creator of the shared memory segment,	*/
	    /* we must also initialize it				*/
	    rec->stats = (mod_clamav_stats *)apr_shm_baseaddr_get(rec->shm);
	    memset(rec->stats, 0, sizeof(mod_clamav_stats));
	    rec->stats->cpu = 0.;
	}
    } else {
	/* initialize the pointer to the shm structure			*/
	rec->stats = (mod_clamav_stats *)apr_shm_baseaddr_get(rec->shm);
    }
    mod_clamav_unlock(rec, r);
} /* clamav_shm */
